package com.dragoninfo.dcuc.app.facade;

import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.service.IServiceResourceService;
import com.dragoninfo.dcuc.app.vo.ServiceResourceVo;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author huangzqa
 * @date 2021/3/1
 **/

@RestController
@RequestMapping(value = "/dcuc/app/serviceResourceFacade")
public class ServiceResourceFacade implements IServiceResourceFacade {

    @Autowired
    private IServiceResourceService serviceResourceService;


    @Override
    public ServiceResource detailByCode(String serviceCode) {
        return serviceResourceService.detailByCode(serviceCode);
    }

    @Override
    public Page<ServiceResource> resourcePages(SearchDTO searchDTO) {
        return serviceResourceService.resourcePages(Searchable.toSearchable(searchDTO));
    }

    @Override
    public List<ServiceResourceDTO> getServiceByServiceCodes(List<String> list) {
        List<ServiceResource> services = serviceResourceService.getServiceByServiceCodes(list);
        return services.stream().map(item -> {
            ServiceResourceDTO dto = new ServiceResourceDTO();
            BeanUtils.copyProperties(item, dto);
            return dto;
        }).collect(Collectors.toList());
    }
    /**
     * 服务资源列表分页查询
     * @return
     */
    @Override
    public Page<ServiceResourceVo> page(SearchDTO searchDTO) {
        return serviceResourceService.page(Searchable.toSearchable(searchDTO));
    }

    /**
     * 服务资源导入
     * @param fileId
     * @return
     */
    @Override
    public ResponseStatus impExcel(String fileId) {
        return serviceResourceService.impExcel(fileId);

    }

    /**
     * 服务资源新增
     * @param serviceResourceDTO
     * @return
     */
    @Override
    public ResponseStatus save(ServiceResourceDTO serviceResourceDTO) {
        return serviceResourceService.saveServiceResource(serviceResourceDTO);
    }
    /**
     * 服务资源详情
     * @param id
     * @return
     */
    @Override
    public ServiceResourceVo detailById(String id) {
        return serviceResourceService.detailById(id);
    }

    /**
     * 服务资源修改
     * @param serviceResourceVo
     */
    @Override
    public ResponseStatus updateServiceResource(ServiceResourceVo serviceResourceVo) {
        return serviceResourceService.updateServiceResource(serviceResourceVo);
    }

    /**
     * 服务资源删除
     * @param id
     * @return
     */
    @Override
    public ResponseStatus deleteById(String id) {
        return serviceResourceService.deleteServiceResourceById(id);
    }


    @Override
    public List<ServiceResourceVo> getServiceByIds(List<String> serviceIds) {
        return serviceResourceService.getServiceByIds(serviceIds);
    }
}
