package com.dragoninfo.dcuc.app.message;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.app.business.IResourceBusiness;
import com.dragoninfo.dcuc.app.cons.ResourceIncrSyncMsgType;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.mq.client.model.entity.MessageInfoDTO;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;

/**
 * 应用服务资源增量同步Kafka消息监听器
 *
 * @author mazq
 * @date 2021/3/10
 */
public class AppResourceSyncListener {

    Logger logger = LoggerFactory.getLogger(AppResourceSyncListener.class);

    @Autowired
    private IResourceBusiness resourceBusiness;

    @KafkaListener(topics = "${dcuc.app.resource-index.app-topic}", containerFactory = "kafkaListenerContainerFactory")
    public void receiveMessage(ConsumerRecord<String, byte[]> record) {
        try {
            byte[] value = record.value();
            if (null == value || value.length == 0) {
                logger.info("receiveMessage value length 0 or null");
                return;
            }
            ObjectInputStream stream = new ObjectInputStream(new ByteArrayInputStream(value));
            MessageInfoDTO messageInfoDTO = (MessageInfoDTO) stream.readObject();
            logger.info("--------AppResourceSyncListener 收到数据，{}", JSON.toJSONString(messageInfoDTO));
            String content = messageInfoDTO.getContent();
            if (StringUtils.isBlank(content)) {
                return;
            }
            JSONObject jsonObject = JSONObject.parseObject(content);
            String appCode = jsonObject.getString("yyxtbh");
            String operateType = jsonObject.getString("sqlx");
            if (StringUtils.isBlank(appCode)) {
                logger.info("appCode is blank");
                return;
            }
            if (ResourceIncrSyncMsgType.APP_UPDATE.equals(operateType) ||
                    ResourceIncrSyncMsgType.APP_REGISTER.equals(operateType)) {
                resourceBusiness.appIncrSync(appCode);
            } else if (ResourceIncrSyncMsgType.APP_STATR.equals(operateType)) {
                resourceBusiness.startApp(appCode);
            } else if (ResourceIncrSyncMsgType.APP_STOP.equals(operateType)) {
                resourceBusiness.stopApp(appCode);
            } else if (ResourceIncrSyncMsgType.APP_CANCEL.equals(operateType)) {
                resourceBusiness.cancelApp(appCode);
            }
        } catch (Exception e) {
            logger.error("receiveMessage error.", e);
        }
    }

}
