package com.dragoninfo.dcuc.app.message;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.app.business.IResourceBusiness;
import com.dragoninfo.dcuc.app.cons.ResourceIncrSyncMsgType;
import com.dragoninfo.dcuc.auth.auth.facade.IServiceAuthResultFacade;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.mq.client.model.entity.MessageInfoDTO;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;

/**
 * 应用服务资源增量同步Kafka消息监听器
 *
 * @author mazq
 * @date 2021/3/10
 */
public class ServiceResourceSyncListener {

    Logger logger = LoggerFactory.getLogger(ServiceResourceSyncListener.class);

    @Autowired
    private IResourceBusiness resourceBusiness;
    @Autowired
    private IServiceAuthResultFacade serviceAuthResultFacade;

    @KafkaListener(topics = "${dcuc.app.resource-index.service-topic}", containerFactory = "kafkaListenerContainerFactory")
    public void receiveMessage(ConsumerRecord<String, byte[]> record) {
        try {
            byte[] value = record.value();
            if (null == value || value.length == 0) {
                logger.info("receiveMessage value length 0 or null");
                return;
            }
            ObjectInputStream stream = new ObjectInputStream(new ByteArrayInputStream(value));
            MessageInfoDTO messageInfoDTO = (MessageInfoDTO) stream.readObject();
            logger.info("-------- ServiceResourceSyncListener 收到数据，{}", JSON.toJSONString(messageInfoDTO));
            String content = messageInfoDTO.getContent();
            if (StringUtils.isBlank(content)) {
                return;
            }
            JSONObject jsonObject = JSONObject.parseObject(content);
            String serviceCode = jsonObject.getString("fwzybsf");
            String operateType = jsonObject.getString("sqlx");
            if (StringUtils.isBlank(serviceCode)) {
                logger.info("serviceCode is blank");
                return;
            }
            if (ResourceIncrSyncMsgType.SERVICE_REGISTER.equals(operateType)
                    || ResourceIncrSyncMsgType.SERVICE_UPDATE.equals(operateType)) {
                resourceBusiness.serviceIncrSync(serviceCode);
            } else if (ResourceIncrSyncMsgType.SERVICE_STATR.equals(operateType)) {
                resourceBusiness.startService(serviceCode);
                //发送新增变更服务通知
                serviceAuthResultFacade.sendAddPermssionServiceUpdate(serviceCode);
            } else if (ResourceIncrSyncMsgType.SERVICE_STOP.equals(operateType)) {
                resourceBusiness.stopService(serviceCode);
                //发送删除变更服务通知
                serviceAuthResultFacade.sendDelPermssionServiceUpdate(serviceCode);
            } else if (ResourceIncrSyncMsgType.SERVICE_CANCEL.equals(operateType)) {
                resourceBusiness.cancelService(serviceCode);
                //发送删除变更服务通知
                serviceAuthResultFacade.sendDelPermssionServiceUpdate(serviceCode);
            }
        } catch (Exception e) {
            logger.error("receiveMessage error.", e);
        }
    }

}
