package com.dragoninfo.dcuc.app.service;

import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragonsoft.duceap.base.entity.metadata.CodeRecord;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.core.service.IBaseService;
import org.springframework.data.domain.Page;

import java.util.List;
import java.util.Map;

/**
 * @author yica
 * @date 2018/8/8
 */
public interface IManufacturerInfoService extends IBaseService<ManufacturerInfo, String> {

    /**
     * 分页查询
     *
     * @param searchDTO 查询条件
     * @return 厂商列表
     */
    Page<ManufacturerInfo> manufacturerInfoPage(SearchDTO searchDTO);

    /**
     * 详情
     *
     * @param id id
     * @return 厂商详情
     */
    ManufacturerInfo manufacturerInfoDetail(String id);

    /**
     * 保存
     *
     * @param manufacturerInfo 厂商信息
     * @return 保存后的厂商信息
     */
    ManufacturerInfo saveManufacturer(ManufacturerInfo manufacturerInfo);

    /**
     * 修改
     *
     * @param manufacturerInfo 厂商信息
     * @return 保存后的厂商信息
     */
    ManufacturerInfo updateManufacturer(ManufacturerInfo manufacturerInfo);

    /**
     * 删除
     *
     * @param id 厂商id
     */
    void del(String id);

    /**
     * 通过厂商code获取厂商
     *
     * @param code 厂商代码
     * @return 厂商信息
     */
    ManufacturerInfo detailByCode(String code);

    /**
     * 根据名称获取厂商
     *
     * @param manufacturerName 厂商名称
     * @return 厂商详情
     */
    ManufacturerInfo getOneByName(String manufacturerName);

    /**
     * 根据名称获取厂商
     *
     * @param code             厂商代码
     * @param manufacturerName 厂商名称
     * @return 厂商详情
     */
    ManufacturerInfo getOneByCodeName(String code, String manufacturerName);

    /**
     * 获取所有厂商代码
     * Value{id},Label{name}
     *
     * @return List<CodeRecord>
     */
    List<CodeRecord> getAllCodeRecord();

    /**
     * 获取所有厂商id名称映射
     *
     * @return Key{id},Value{name} 的 Map
     */
    Map<String, String> getAllIdNameMap();

    /**
     * 根据code获取所有厂商
     * @param list 厂商代码集合
     * @return
     */
    List<ManufacturerInfo> getByCodes(List<String> list);
}
