package com.dragoninfo.dcuc.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.entity.ManufacturerInfo;
import com.dragoninfo.dcuc.app.mapper.ManufacturerInfoMapper;
import com.dragoninfo.dcuc.app.service.IManufacturerInfoService;
import com.dragonsoft.duceap.base.entity.metadata.CodeRecord;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.mybaitsplus.core.services.BaseMybatisService;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author huangzqa
 * @date 2020/10/26
 */
@CacheConfig(cacheNames = "DCUC-APP-MANU")
@Service
public class ManufacturerInfoServiceImpl extends BaseMybatisService<ManufacturerInfo, String> implements IManufacturerInfoService {

    private ManufacturerInfoMapper manufacturerInfoMapper;


    public ManufacturerInfoServiceImpl(ManufacturerInfoMapper manufacturerInfoMapper) {
        super(manufacturerInfoMapper);
        this.manufacturerInfoMapper = manufacturerInfoMapper;
    }

    @Cacheable(key = "targetClass +methodName +#p0")
    @Override
    public Page<ManufacturerInfo> manufacturerInfoPage(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        searchable.addSearchFilter("deleted", SearchOperator.eq, BooleanEnum.FALSE.getValue());
        return manufacturerInfoMapper.pagingBySearchable(searchable);
    }

    @Cacheable(key = "targetClass +#p0")
    @Override
    public ManufacturerInfo manufacturerInfoDetail(String id) {
        return manufacturerInfoMapper.selectById(id);
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ManufacturerInfo saveManufacturer(ManufacturerInfo info) {
        info.setDeleted(BooleanEnum.FALSE.getValue());
        manufacturerInfoMapper.insert(info);
        return info;
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ManufacturerInfo updateManufacturer(ManufacturerInfo info) {
        manufacturerInfoMapper.updateById(info);
        return info;
    }

    @CacheEvict(allEntries = true)
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void del(String id) {
        ManufacturerInfo manufacturerInfo = manufacturerInfoMapper.selectById(id);

        // 有使用  @TableLogic 会进行逻辑删除
        manufacturerInfoMapper.deleteById(manufacturerInfo);
    }

    @Cacheable(key = "targetClass +methodName +#p0")
    @Override
    public ManufacturerInfo detailByCode(String code) {
        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ManufacturerInfo::getCode, code)
                .eq(ManufacturerInfo::getDeleted, BooleanEnum.FALSE.getValue());

        return manufacturerInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass +methodName +#p0")
    @Override
    public ManufacturerInfo getOneByName(String manufacturerName) {

        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ManufacturerInfo::getManufacturerName, manufacturerName)
                .eq(ManufacturerInfo::getDeleted, BooleanEnum.FALSE.getValue());

        return manufacturerInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass +methodName +#p0+p1")
    @Override
    public ManufacturerInfo getOneByCodeName(String code, String manufacturerName) {
        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ManufacturerInfo::getCode, code)
                .eq(ManufacturerInfo::getManufacturerName, manufacturerName);
        return manufacturerInfoMapper.selectOne(queryWrapper);
    }

    @Cacheable(key = "targetClass +methodName")
    @Override
    public List<CodeRecord> getAllCodeRecord() {

        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ManufacturerInfo::getId, ManufacturerInfo::getManufacturerName)
                .eq(ManufacturerInfo::getDeleted, BooleanEnum.FALSE.value);

        List<ManufacturerInfo> manufacturerInfoList = manufacturerInfoMapper.selectList(queryWrapper);

        return manufacturerInfoList.stream()
                .map(item -> {
                    String id = item.getId();
                    String manufacturerName = item.getManufacturerName();
                    CodeRecord codeRecord = new CodeRecord();
                    codeRecord.setValue(id);
                    codeRecord.setLabel(manufacturerName);
                    return codeRecord;
                })
                .collect(Collectors.toList());
    }

    @Cacheable(key = "targetClass +methodName")
    @Override
    public Map<String, String> getAllIdNameMap() {
        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(ManufacturerInfo::getId, ManufacturerInfo::getManufacturerName);

        List<ManufacturerInfo> manufacturerInfoList = manufacturerInfoMapper.selectList(queryWrapper);

        return manufacturerInfoList.stream()
                .collect(Collectors.toMap(ManufacturerInfo::getId, ManufacturerInfo::getManufacturerName));
    }

    @Cacheable(key = "targetClass +methodName +#p0")
    @Override
    public List<ManufacturerInfo> getByCodes(List<String> list) {
        LambdaQueryWrapper<ManufacturerInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select()
                .in(ManufacturerInfo::getCode);
        return manufacturerInfoMapper.selectList(queryWrapper);
    }

}
