package com.dragoninfo.dcuc.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.cons.ServiceStatusCons;
import com.dragoninfo.dcuc.app.dto.ServiceResourceDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ServiceResource;
import com.dragoninfo.dcuc.app.mapper.ServiceResourceMapper;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragoninfo.dcuc.app.service.IServiceResourceService;
import com.dragoninfo.dcuc.app.vo.ServiceResourceVo;
import com.dragoninfo.dcuc.auth.auth.enumresources.YesNotEnum;
import com.dragoninfo.dcuc.common.utils.DcucBeanUtil;
import com.dragoninfo.dcuc.duceap.enums.ServiceProvideWayEnum;
import com.dragoninfo.dcuc.duceap.enums.ServiceResourceTypeEnum;
import com.dragoninfo.dcuc.duceap.facade.IDuceapUploadFacade;
import com.dragoninfo.dcuc.duceap.upload.dto.DocContentDTO;
import com.dragoninfo.dcuc.user.facade.org.IOrgInfoFacade;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import com.dragonsoft.duceap.mybaitsplus.core.services.BaseMybatisService;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author huangzqa
 * @date 2021/3/1
 **/
@Transactional(rollbackFor = Exception.class)
@Service
public class ServiceResourceServiceImpl extends BaseMybatisService<ServiceResource, String> implements IServiceResourceService {


    private final ServiceResourceMapper serviceResourceMapper;

    @Autowired
    private IApplyInfoService applyInfoService;

    @Autowired
    private IOrgInfoFacade orgInfoFacade;

    @Resource
    private IDuceapUploadFacade uploadhandler;

    public ServiceResourceServiceImpl(ServiceResourceMapper serviceResourceMapper) {
        super(serviceResourceMapper);
        this.serviceResourceMapper = serviceResourceMapper;
    }

    @Override
    public ServiceResource detailByCode(String serviceCode) {

        LambdaQueryWrapper<ServiceResource> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(ServiceResource::getServiceCode, serviceCode);

        return serviceResourceMapper.selectOne(queryWrapper);
    }
    @Override
    public Page<ServiceResource> resourcePages(Searchable searchable){
        searchable.addSort(Sort.Direction.DESC,"create_time");
        Page<ServiceResource> serviceResources = serviceResourceMapper.pagingBySearchable(searchable);
        return serviceResources;
    }

    @Override
    public List<ServiceResource> getServiceByServiceCodes(List<String> list) {
        LambdaQueryWrapper<ServiceResource> query = Wrappers.lambdaQuery();
        query.select()
                .in(ServiceResource::getServiceCode, list)
                .eq(ServiceResource::getDeleted, BooleanEnum.FALSE.getValue());
        return serviceResourceMapper.selectList(query);
    }

    @Override
    public boolean updateById(ServiceResourceDTO dto) {
        ServiceResource serviceResource = new ServiceResource();
        BeanUtils.copyProperties(dto, serviceResource);
        serviceResource.setUpdateTime(new Date());
        serviceResourceMapper.updateById(serviceResource);
        return true;
    }
    /**
     * 服务资源列表分页查询
     * @return
     */
    @Override
    public Page<ServiceResourceVo> page(Searchable searchable) {
        //如果查询条件中含有应用名称
        Condition appNameCondition = searchable.getSearchFilter("appName", SearchOperator.like);
        String appName = null;
        Map<String,ApplyInfo> applyInfoMap = null;
        if(null != appNameCondition && StringUtils.isNotBlank(appName = appNameCondition.getValue().toString())) {
            List<ApplyInfo> appList = applyInfoService.getAppByAppNameLike(appName);
            if(CollectionUtils.isEmpty(appList)) {
                return new PageImpl<>(new ArrayList<>(), searchable.getPage(), 0L);
            }
            searchable.removeSearchFilter("appName", SearchOperator.like);
            applyInfoMap = appList.stream().collect(Collectors.toMap(ApplyInfo::getApplyCode,item->item,(old,last)->last));
            searchable.addSearchFilter("app_code",SearchOperator.in, applyInfoMap.keySet());
        }
        searchable.addSearchFilter("deleted", SearchOperator.eq, BooleanEnum.FALSE.value);
        searchable.addSort(Sort.Direction.DESC,"create_time");
        Page<ServiceResource> serviceResources = serviceResourceMapper.pagingBySearchable(searchable);
        List<ServiceResourceVo> result = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(serviceResources.getContent())){
            for (ServiceResource serviceResource :serviceResources.getContent()){
                ServiceResourceVo serviceResourceVo = new ServiceResourceVo();
                BeanUtils.copyProperties(serviceResource, serviceResourceVo);
                result.add(serviceResourceVo);
            }
            //设置应用名称
            batchAppendAppName(applyInfoMap, result);

        }
        return new PageImpl<>(result, searchable.getPage(), serviceResources.getTotalElements());
    }

    private void batchAppendAppName(Map<String, ApplyInfo> applyInfoMap, List<ServiceResourceVo> result) {
        Map<String, ApplyInfo> appMap;
        if(null == applyInfoMap){
            List<String> appCodes = result.stream().map(ServiceResourceVo::getAppCode).distinct().collect(Collectors.toList());
            List<ApplyInfo> applyInfoList = applyInfoService.getListByCodes(appCodes);
            appMap = applyInfoList.stream().collect(Collectors.toMap(ApplyInfo::getApplyCode,item->item,(old,last)->last));
        } else {
            appMap = applyInfoMap;
        }
        result.forEach(item->{
            String appCode = item.getAppCode();
            ApplyInfo applyInfo = appMap.get(appCode);
            if(null != applyInfo){
                item.setAppName(applyInfo.getApplyName());
            }
        });
    }


    /**
     * 服务资源导入
     * @param fileId
     * @return
     */
    @Override
    public ResponseStatus impExcel(String fileId) {
        try {
            DocContentDTO fileContent = uploadhandler.getFileContent(fileId);
            byte[] fileData = fileContent.getFileData();
            InputStream inputStream = new ByteArrayInputStream(fileData);
            List<ServiceResource> list = loadServiceResourceInfo(inputStream);
            if(list.size() > 1000) {
                return ResponseStatus.fail("300","导入数据超过1000行");
            }
            saveAll(list);
        }catch (IOException e) {
            return ResponseStatus.fail("300","导入文件类型错误");
        } catch (NumberFormatException e) {
            return ResponseStatus.fail("300",e.getMessage());
        } catch (Exception e) {
            return ResponseStatus.fail("300","导入失败");
        }
        return ResponseStatus.success();
    }

    /**
     * 服务资源新增
     * @param serviceResourceDTO
     * @return
     */
    @Override
    public ResponseStatus saveServiceResource(ServiceResourceDTO serviceResourceDTO) {
        //判断标识符不能重复
        ServiceResource resource = detailByCode(serviceResourceDTO.getServiceCode());
        if (resource != null){
            return ResponseStatus.fail("300", "服务资源标识符已存在");
        }
        //判断是否为正确的应用
        ApplyInfo applyInfo = applyInfoService.getAppByCode(serviceResourceDTO.getAppCode());
        if (applyInfo == null){
            return ResponseStatus.fail("300", "服务提供应用系统不存在");
        }
        ServiceResource serviceResource = new ServiceResource();
        BeanUtils.copyProperties(serviceResourceDTO, serviceResource);
        serviceResource.setCreateTime(new Date());
        serviceResource.setDeleted(YesNotEnum.NO.getValue());
        serviceResource.setServiceStatus(ServiceStatusCons.STATUS_START);
        save(serviceResource);
        return ResponseStatus.success();
    }

    /**
     * 服务资源详情
     * @param id
     * @return
     */
    @Override
    public ServiceResourceVo detailById(String id) {
        ServiceResource serviceResource = serviceResourceMapper.selectById(id);
        ServiceResourceVo resourceVo = new ServiceResourceVo();
        if (serviceResource != null){
            BeanUtils.copyProperties(serviceResource, resourceVo);
            appendAppName(resourceVo);
        }
        return resourceVo;
    }

    /**
     * 服务资源修改
     * @param serviceResourceVo
     * @return
     */
    @Override
    public ResponseStatus updateServiceResource(ServiceResourceVo serviceResourceVo) {
        ServiceResource serviceResource = serviceResourceMapper.selectById(serviceResourceVo.getId());
        if (serviceResource != null){
            BeanUtils.copyProperties(serviceResourceVo, serviceResource);
            serviceResource.setUpdateTime(new Date());
            serviceResourceMapper.updateById(serviceResource);
        }
        return new ResponseStatus();
    }

    /**
     * 服务资源删除
     * @param id
     * @return
     */
    @Override
    public ResponseStatus deleteServiceResourceById(String id) {
        ServiceResource serviceResource = serviceResourceMapper.selectById(id);
        if (serviceResource != null){
            serviceResource.setDeleted(BooleanEnum.TRUE.getValue());
            serviceResource.setUpdateTime(new Date());
            serviceResourceMapper.updateById(serviceResource);
            serviceResourceMapper.deleteById(id);
        }
        return new ResponseStatus();
    }

    @Override
    public Long count(Date startTime, Date endTime) {
        LambdaQueryWrapper<ServiceResource> query = Wrappers.lambdaQuery();
        if (null != startTime) {
            query.ge(ServiceResource::getCreateTime, startTime);
        }
        if (null != endTime) {
            query.le(ServiceResource::getCreateTime, endTime);
        }

        return Long.valueOf(serviceResourceMapper.selectCount(query));
    }

    @Override
    public List<ServiceResourceVo> getServiceByIds(List<String> serviceIds) {
        if (CollectionUtils.isEmpty(serviceIds)) {
            return Collections.emptyList();
        }
        LambdaQueryWrapper<ServiceResource> wrapper = Wrappers.lambdaQuery();
        wrapper.in(ServiceResource::getId, serviceIds);
        List<ServiceResource> serviceResources = serviceResourceMapper.selectList(wrapper);
        return DcucBeanUtil.createCopyToObjectList(serviceResources, ServiceResourceVo.class);
    }

    /**
     * 构建服务资源数组
     * @param inputStream
     * @return
     * @throws IOException
     */
    public List<ServiceResource> loadServiceResourceInfo(InputStream inputStream) throws IOException {
        List<ServiceResource> result = new ArrayList<>();
        //根据指定的文件输入流导入Excel从而产生Workbook对象
        Workbook wb0 = new HSSFWorkbook(inputStream);
        //获取Excel文档中的第一个表单
        Sheet sht0 = wb0.getSheetAt(0);
        List<String> codeList = new ArrayList<>();
        for (Row r : sht0) {
            if (r.getRowNum() == 0) {
                checkRowServiceResourceTitle(r);
                continue;
            }else {
                //循环每行的每一列
                ServiceResource resource = new ServiceResource();

                boolean isAllEmpty = true;
                for (int i = 0; i < 8; i++){
                    Cell cell = r.getCell(i);
                    if (cell != null){
                        r.getCell(i).setCellType(Cell.CELL_TYPE_STRING);
                    }
                    if (cell != null && StringUtils.isNotBlank(cell.getStringCellValue())){
                        isAllEmpty = false;
                        break;
                    }
                }
                if (!isAllEmpty){
                    for (int i = 0; i < 8; i++) {
                        Cell cell = r.getCell(i);
                        if (cell != null){
                            r.getCell(i).setCellType(Cell.CELL_TYPE_STRING);
                        }
                        checkServiceResourceCellData(cell, i, r.getRowNum());
                        checkServiceResourceCellDataValid(cell, i, r.getRowNum(), resource, codeList);
                    }
                    resource.setCreateTime(new Date());
                    resource.setDeleted(YesNotEnum.NO.getValue());
                    resource.setServiceStatus(ServiceStatusCons.STATUS_START);
                    result.add(resource);
                }

            }

        }
        return result;
    }

    /**
     * 校验参数和设置参数
     * @param cell
     * @param num
     * @param row
     * @param resource
     */
    private void checkServiceResourceCellDataValid(Cell cell, int num, int row, ServiceResource resource, List<String> codeList) {
        if (cell == null) {
            return;
        }
        String value = cell.getStringCellValue();
        switch (num) {
            case 0:
                if (value.length() > 100) {
                    throw new NumberFormatException("第" + (row + 1) + "行'服务资源标识符'长度大于100！");
                }
                //判断标识符不能重复
                ServiceResource serviceResource = detailByCode(value);
                if (serviceResource != null){
                    throw new NumberFormatException("第" + (row + 1) + "行'服务资源标识符'已存在！");
                }
                if (codeList.contains(value)){
                    throw new NumberFormatException("第" + (row + 1) + "行'服务资源标识符'重复！");
                }
                codeList.add(value);
                resource.setServiceCode(value);
                break;
            case 1:
                if (value.length() > 500) {
                    throw new NumberFormatException("第" + (row + 1) + "行'服务资源名称'长度大于500！");
                }
                resource.setServiceName(value);
                break;
            case 2:
                String resourceType = "";
                for (ServiceResourceTypeEnum typeEnum : ServiceResourceTypeEnum.values()) {
                    if(typeEnum.getLabel().equals(value)){
                        resourceType = typeEnum.getValue();
                        break;
                    }
                }
                if(StringUtils.isBlank(resourceType)) {
                    throw new NumberFormatException("第" + (row + 1) + "行'服务类型'填写不正确！");
                }
                resource.setServiceType(resourceType);
                break;
            case 3:
                //判断服务提供方式的枚举
                String wayCode = "";
                for (ServiceProvideWayEnum wayEnum: ServiceProvideWayEnum.values()){
                    if (value.equals(wayEnum.getLabel())){
                        wayCode = wayEnum.getValue();
                        break;
                    }
                }
                if (StringUtils.isBlank(wayCode) ){
                    throw new NumberFormatException("第" + (row + 1) + "行'服务提供方式'填写不正确！");
                }

                if (wayCode.length() > 20) {
                    throw new NumberFormatException("第" + (row + 1) + "行'服务提供方式'长度大于20！");
                }

                resource.setServiceProvideWay(wayCode);
                break;
            case 4:
                if (StringUtils.isNotBlank(value)){
                    String code = "";
                    for (YesNotEnum yesNotEnum: YesNotEnum.values()){
                        if (value.equals(yesNotEnum.getLabel())){
                            code = yesNotEnum.getValue();
                            continue;
                        }
                    }
                    //是否自用服务
                    resource.setMustSelf(code);
                }
                break;
            case 5:
                if (value.length() > 500) {
                    throw new NumberFormatException("第" + (row + 1) + "行'服务访问地址'长度大于500！");
                }
                resource.setServiceUrl(value);
                break;
            case 6:
                //判断是否为正确的应用
                ApplyInfo applyInfo = applyInfoService.getOneByName(value);
                if (applyInfo == null){
                    throw new NumberFormatException("第" + (row + 1) + "行'服务提供应用系统名称'不存在！");
                }
                resource.setAppCode(applyInfo.getApplyCode());
                break;
            case 7:
                resource.setRemark(value);
                break;
            default:
                break;

        }
    }


    /**
     * 校验数据是否为空
     * @param cell
     * @param i
     * @param row
     */
    private static void checkServiceResourceCellData(Cell cell, int i, int row) {
        if (!(cell == null || StringUtils.isBlank(cell.getStringCellValue()))) {
            return;
        }
        switch (i) {
            case 0:
                throw new NumberFormatException("第" + (row + 1) + "行'服务资源标识'为空！");
            case 1:
                throw new NumberFormatException("第" + (row + 1) + "行'服务资源名称'为空！");
            case 2:
                throw new NumberFormatException("第" + (row + 1) + "行'服务类型'为空！");
            case 3:
                throw new NumberFormatException("第" + (row + 1) + "行'服务提供方式'为空！");
            case 5:
                throw new NumberFormatException("第" + (row + 1) + "行'服务访问地址'为空！");
            case 6:
                throw new NumberFormatException("第" + (row + 1) + "行'服务提供应用系统名称'为空！");
            default:
                break;
        }

    }

    /**
     * 校验列头
     * @param r
     */
    private static void checkRowServiceResourceTitle(Row r) {
        if (!r.getCell(0).getStringCellValue().equals("服务资源标识")) {
            throw new NumberFormatException("缺少'服务资源标识'列！");
        }
        if (!r.getCell(1).getStringCellValue().equals("服务资源名称")) {
            throw new NumberFormatException("缺少'服务资源名称'列！");
        }
        if (!r.getCell(2).getStringCellValue().equals("服务类型")) {
            throw new NumberFormatException("缺少'服务类型'列！");
        }
        if (!r.getCell(3).getStringCellValue().equals("服务提供方式")) {
            throw new NumberFormatException("缺少'服务提供方式'列！");
        }
        if (!r.getCell(4).getStringCellValue().equals("是否自用服务")) {
            throw new NumberFormatException("缺少'是否自用服务'列！");
        }
        if (!r.getCell(5).getStringCellValue().equals("服务访问地址")) {
            throw new NumberFormatException("缺少'服务访问地址'列！");
        }
        if (!r.getCell(6).getStringCellValue().equals("服务提供应用系统名称")) {
            throw new NumberFormatException("缺少'服务提供应用系统名称'列！");
        }
        if (!r.getCell(7).getStringCellValue().equals("服务资源描述")) {
            throw new NumberFormatException("缺少'服务资源描述'列！");
        }
    }

    /**
     * 设置应用名称
     * @param serviceResourceVo
     */
    private void appendAppName(ServiceResourceVo serviceResourceVo) {
        if(StringUtils.isBlank(serviceResourceVo.getAppCode())){
            return;
        }
        ApplyInfo applyInfo = applyInfoService.getAppByCode(serviceResourceVo.getAppCode());
        if(null != applyInfo){
            serviceResourceVo.setAppName(applyInfo.getApplyName());
        }

    }
}
