package com.dragoninfo.dcuc.app.strategy;


import cn.hutool.core.bean.BeanUtil;
import com.dragoninfo.dcuc.app.dto.AppResourceDTO;
import com.dragoninfo.dcuc.app.dto.ApplyInfoDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.entity.ApplyOauth;
import com.dragoninfo.dcuc.app.enumresources.ResourceTypeEnum;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragoninfo.dcuc.app.service.IApplyOauthService;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author mazq
 * @Description
 * @create 2020-11-25 14:21
 */
@Component
public class AppResourceStrategy implements ResourceOperateStrategy<AppResourceDTO, ApplyInfo> {

    @Autowired
    private IApplyInfoService applyInfoService;

    @Autowired
    private IApplyOauthService applyOauthService;

    @Override
    public ApplyInfo insertRemoteSource(AppResourceDTO appResourceDTO) {
        ApplyInfo applyInfo;
        ApplyInfo exist = applyInfoService.getAppByCode(appResourceDTO.getAppCode());
        if (null == exist) {
            applyInfo = insertApplyInfo(appResourceDTO);
        } else {
            applyInfo = updateApplyInfo(exist, appResourceDTO);
        }
        return applyInfo;
    }

    @Override
    public boolean canHandle(String resourceType) {
        return ResourceTypeEnum.TJ_APP.getResourceId().equals(resourceType);
    }

    private ApplyInfo insertApplyInfo(AppResourceDTO dto) {
        ApplyInfo applyInfo = convertToApplyInfo(dto);
        //删除的应用,不插入数据库
        if (BooleanEnum.TRUE.getValue().equals(dto.getDeleted())) {
            return applyInfo;
        }
        String appSecret = dto.getAppSecret();
        if (StringUtils.isBlank(appSecret)) {
            String secretKey = getApiKeys(applyInfo.getApplyCode());
            String apiKey = getApiKeys(applyInfo.getApplyCode());
            applyInfo.setSecretKey(secretKey);
            applyInfo.setApiKey(apiKey);
        }
        ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
        BeanUtil.copyProperties(applyInfo, applyInfoDTO);
        applyInfoService.saveAppIy(applyInfoDTO);
        applyOauthService.saveOauth(applyInfo);
        return applyInfo;
    }

    private ApplyInfo updateApplyInfo(ApplyInfo applyInfo, AppResourceDTO dto) {
        //是需要删除的应用
        if (BooleanEnum.TRUE.getValue().equals(dto.getDeleted())) {
            applyInfoService.del(applyInfo.getId());
            return applyInfo;
        }
        ApplyInfo infoNew = convertToApplyInfo(dto);
        BeanUtil.copyProperties(infoNew, applyInfo, "id", "apiKey", "secretKey");
        //更新Secret
        String appSecret = dto.getAppSecret();
        ApplyOauth auth = applyOauthService.getByAppId(applyInfo.getId());
        if (null == auth && StringUtils.isNotBlank(appSecret)) {
            insertAppOauth(applyInfo, appSecret);
        } else if (null != auth && StringUtils.isNotBlank(appSecret)) {
            String secretKey = auth.getSecretKey();
            if (!appSecret.equals(secretKey)) {
                updateAppOauth(applyInfo, appSecret);
            }
        }
        ApplyInfoDTO applyInfoDTO = new ApplyInfoDTO();
        BeanUtil.copyProperties(applyInfo, applyInfoDTO);
        applyInfoService.updateApply(applyInfoDTO);
        return applyInfo;
    }

    private void updateAppOauth(ApplyInfo applyInfo, String appSecret) {
        applyInfo.setSecretKey(appSecret);
        applyInfo.setApiKey(appSecret);
        applyOauthService.updateOauth(applyInfo);
    }

    private void insertAppOauth(ApplyInfo applyInfo, String appSecret) {
        applyInfo.setSecretKey(appSecret);
        applyInfo.setApiKey(appSecret);
        applyOauthService.saveOauth(applyInfo);
    }


    private ApplyInfo convertToApplyInfo(AppResourceDTO dto) {
        String appCode = dto.getAppCode();
        ApplyInfo applyInfo = new ApplyInfo();
        applyInfo.setApplyCode(appCode);
        applyInfo.setApplyName(dto.getAppName());
        applyInfo.setOrgName(dto.getBuildUnit());
        applyInfo.setManagerOrgName(dto.getManagerUnit());
        applyInfo.setJgFzr(dto.getBuildPrincipal());
        applyInfo.setJgFzrPhone(dto.getBuildPrincipalPhone());
        applyInfo.setFzr(dto.getContractPrincipal());
        applyInfo.setFzrPhone(dto.getContractPrincipalPhone());
        applyInfo.setManufacturerName(dto.getContractUnit());
        applyInfo.setDeleted(dto.getDeleted());
        applyInfo.setApplyUrl(dto.getAppUrl());
        applyInfo.setApplyCategory(dto.getApplyCategory());
        applyInfo.setPoliceSpecies(dto.getPoliceSpecies());
        applyInfo.setRegistrationTime(dto.getRegistrationTime());
        applyInfo.setApplyStatus(dto.getAppStatus());
        applyInfo.setSecretKey(dto.getAppSecret());
        applyInfo.setApiKey(dto.getAppSecret());
        applyInfo.setAppOpsDepName(dto.getAppOpsDepName());
        applyInfo.setOrgCode(dto.getBuildUnitCode());
        applyInfo.setLineTime(dto.getLineTime());
        applyInfo.setAppEnglishName(dto.getAppEnglishName());
        return applyInfo;
    }

    private String getApiKeys(String value) {
        StringBuilder num = new StringBuilder();
        for (int i = 0; i < 8; i++) {
            int random = (int) (Math.random() * 10);
            num.append(random);
        }
        return value + num;
    }

}
