package com.dragoninfo.dcuc.app.business.impl;

import com.dragoninfo.dcuc.app.config.FhAppSyncProperties;
import com.dragoninfo.dcuc.app.dto.appindex.FhAppIndexRespDTO;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.common.http.SkipSslVerificationHttpRequestFactory;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.user.entity.org.OrgInfo;
import com.dragoninfo.dcuc.user.facade.org.IOrgInfoFacade;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.ObjectUtils;
import com.github.dreamhead.moco.HttpServer;
import com.github.dreamhead.moco.Runner;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.platform.commons.util.StringUtils;
import org.mockito.Mockito;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.github.dreamhead.moco.Moco.*;

/**
 * @author mazq
 * @date 2023/7/20
 */
public class FhAppSyncBusinessImplTerst {

    public static FhAppSyncBusinessImpl fhAppSyncBusiness;

    public static Runner runner;

    public static String address = "/appmarket/app/resources/QueryAppResource";

    @BeforeClass
    public static void before() {

        // 远程调用服务
        HttpServer httpServer = httpServer(19988, log());
        httpServer.request(by(uri(address)))
                .response(pathResource("fh/app.json"));
        runner = Runner.runner(httpServer);
        runner.start();

        fhAppSyncBusiness = new FhAppSyncBusinessImpl();
        FhAppSyncProperties fhAppSyncProperties = new FhAppSyncProperties();
        // 配置文件
        fhAppSyncProperties.setAddress("http://127.0.0.1:19988" + address);
        fhAppSyncBusiness.setFhAppSyncProperties(fhAppSyncProperties);
        // 机构查询
        IOrgInfoFacade orgInfoFacade = Mockito.spy(IOrgInfoFacade.class);
        List<OrgInfo> orgInfos = new ArrayList<>();
        OrgInfo orgInfo = new OrgInfo();
        orgInfo.setCode("120000000000");
        orgInfo.setFullName("天津市GAJ");
        orgInfos.add(orgInfo);
        Mockito.when(orgInfoFacade.getOrgsByCodes(Mockito.anyList()))
                .thenReturn(orgInfos);
        fhAppSyncBusiness.setOrgInfoFacade(orgInfoFacade);

        // restTemplate
        SkipSslVerificationHttpRequestFactory skipSslVerificationHttpRequestFactory
                = new SkipSslVerificationHttpRequestFactory();
        skipSslVerificationHttpRequestFactory.setReadTimeout(5000);
        skipSslVerificationHttpRequestFactory.setConnectTimeout(5000);
        RestTemplate restTemplate = new RestTemplate(skipSslVerificationHttpRequestFactory);
        fhAppSyncBusiness.setRestTemplate(restTemplate);
    }

    @AfterClass
    public static void after() {
        runner.stop();
    }

    @Test
    public void appSyncTest() {
        ResponseDTO<FhAppIndexRespDTO> appResource = fhAppSyncBusiness.getAppResource();
        FhAppIndexRespDTO result = ResponseUtil.getResult(appResource);
        ResponseDTO<List<ApplyInfo>> appInfoResp = fhAppSyncBusiness.parseResp(result);
        List<ApplyInfo> applyInfos = ResponseUtil.getResult(appInfoResp);
        applyInfos.forEach(e -> {
            String orgCode = e.getOrgCode();
            String orgName = e.getOrgName();
            String managerOrgCode = e.getManagerOrgCode();
            String managerOrgName = e.getManagerOrgName();
            String applyCode = e.getApplyCode();
            String applyStatus = e.getApplyStatus();
            String applyName = e.getApplyName();
            String lineTime = e.getLineTime();
            Date registrationTime = e.getRegistrationTime();
            String apiKey = e.getApiKey();
            String secretKey = e.getSecretKey();
            Assert.assertEquals("120000000000", orgCode);
            Assert.assertEquals("120000000000", managerOrgCode);
            Assert.assertEquals("天津市GAJ", orgName);
            Assert.assertEquals("天津市GAJ", managerOrgName);
            Assert.assertEquals("0", applyStatus);
            Assert.assertTrue(StringUtils.isNotBlank(applyCode));
            Assert.assertTrue(StringUtils.isNotBlank(applyName));
            Assert.assertTrue(StringUtils.isNotBlank(lineTime));
            Assert.assertTrue(ObjectUtils.isNotEmpty(registrationTime));
            Assert.assertTrue(StringUtils.isNotBlank(apiKey));
            Assert.assertTrue(StringUtils.isNotBlank(secretKey));
        });
    }

}
