package com.dragoninfo.dcuc.app.service.impl;

import cn.hutool.json.JSONUtil;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.service.IApplyInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.Page;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @author huangzqa
 * @date 2020/10/28
 */
@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
@Rollback()
public class ApplyInfoServiceImplTest {

    @Autowired
    private IApplyInfoService applyInfoService;

    @Test
    public void applyDetail() {
        String id = "2c90e944668a66c001668aff5de00000";
        ApplyInfo applyInfo = applyInfoService.applyDetail(id);

        System.out.println(JSONUtil.toJsonStr(applyInfo));
        Assert.assertNotNull(applyInfo);
    }

    @Test
    public void applyInfoPage() {
        SearchDTO searchDTO = new SearchDTO();
        searchDTO.setSize("10");
        searchDTO.setPage("1");
        Page<ApplyInfo> applyInfoPage = applyInfoService.applyInfoPage(searchDTO);
        System.out.println(JSONUtil.toJsonStr(applyInfoPage));
    }

    @Test
    public void saveAppIy() {
    }

    @Test
    public void updateApply() {
    }

    @Test
    public void del() {
        String id = "2c90e944668a66c001668aff5de00000";
        applyInfoService.del(id);

        ApplyInfo applyInfo = applyInfoService.applyDetail(id);

        Assert.assertNull(applyInfo);

    }

    @Test
    public void getNewSort() {

        String newSort = applyInfoService.getNewSort();

        Assert.assertNotNull(newSort);
        System.out.println(newSort);
    }

    @Test
    public void startOrStop() {
        String id = "2c90e944668a66c001668aff5de00000";

        ApplyInfo one = applyInfoService.getOne(id);

        String applyStatus = one.getApplyStatus();

        System.out.println(applyStatus);
        String message = applyInfoService.startOrStop(id);

        System.out.println(message);

        ApplyInfo newOne = applyInfoService.getOne(id);
        String newApplyStatus = newOne.getApplyStatus();

        System.out.println(newApplyStatus);

        Assert.assertNotEquals(applyStatus, newApplyStatus);
    }

    @Test
    public void sortApp() {
        String id = "2c90e944668a66c001668aff5de00000";
        String targetId = "2c90e944668a66c001668b710d850002";

        ApplyInfo one = applyInfoService.getOne(id);
        ApplyInfo targetOne = applyInfoService.getOne(targetId);

        String oneSort = one.getSort();
        String targetOneSort = targetOne.getSort();

        applyInfoService.sortApp(id, targetId);

        ApplyInfo newOne = applyInfoService.getOne(id);
        ApplyInfo newTargetOne = applyInfoService.getOne(targetId);

        String newOneSort = newOne.getSort();
        String newTargetOneSort = newTargetOne.getSort();

        Assert.assertEquals(oneSort, newTargetOneSort);
        Assert.assertEquals(targetOneSort, newOneSort);
    }

    @Test
    public void codeConvertToId() {

        String ergebs = applyInfoService.codeConvertToId("ergebs");
        System.out.println(ergebs);
        Assert.assertEquals(ergebs, "2c90e944668a66c001668aff5de00000");

    }

    @Test
    public void convertIdsToNames() {
        String projects = "4028178d6b86fcda016b9ca14d9b0008,4028178d6b86fcda016b913a182d0001,4028178d6b86fcda016b9ca299f0000a";

        String names = applyInfoService.convertIdsToNames(projects);
        System.out.println(names);

        Assert.assertNotNull(names);

        String names1 = applyInfoService.convertIdsToNames("");
        System.out.println(names1);
    }

    @Test
    public void getAppByCode() {
        ApplyInfo appByCode = applyInfoService.getAppByCode("ergebs");
        System.out.println(JSONUtil.toJsonStr(appByCode));
    }

    @Test
    public void isRepeat() {
        ApplyInfo applyInfo = new ApplyInfo();
        applyInfo.setApplyCode("wrsapp");
        applyInfo.setApplyName(UUID.randomUUID().toString());

        ResponseStatus repeat = applyInfoService.isRepeat(applyInfo);

        Assert.assertEquals(repeat.getMessage(), "应用代码已存在");

        applyInfo.setApplyCode(UUID.randomUUID().toString());
        applyInfo.setApplyName("wrs-app");

        ResponseStatus repeat1 = applyInfoService.isRepeat(applyInfo);

        Assert.assertEquals(repeat1.getMessage(), "应用名称已存在");

        ApplyInfo applyInfo1 = applyInfoService.getAppByCode("wrsapp");
        applyInfo1.setApplyCode("test20190705");

        ResponseStatus repeat2 = applyInfoService.isRepeat(applyInfo1);
        Assert.assertEquals(repeat2.getMessage(), "应用代码已存在");

        applyInfo1.setApplyName("test20190705");

        ResponseStatus repeat3 = applyInfoService.isRepeat(applyInfo1);
        Assert.assertEquals(repeat3.getMessage(), "应用名称已存在");


        applyInfo.setApplyName(UUID.randomUUID().toString());
        applyInfo.setApplyCode(UUID.randomUUID().toString());
        ResponseStatus repeat4 = applyInfoService.isRepeat(applyInfo);
        Assert.assertEquals(repeat4.getStatusCode(), "200");


        applyInfo1.setApplyName(UUID.randomUUID().toString());
        applyInfo1.setApplyCode(UUID.randomUUID().toString());
        ResponseStatus repeat5 = applyInfoService.isRepeat(applyInfo1);
        Assert.assertEquals(repeat5.getStatusCode(), "200");

    }

    @Test
    public void fzxmList() {
        String userId = "2c90e9666903bbed016904898cf30000";
        List<ApplyInfo> applyInfos = applyInfoService.fzxmList(userId);

        for (ApplyInfo applyInfo : applyInfos) {
            String fzr = applyInfo.getFzr();
            Assert.assertEquals(fzr, userId);
        }

        System.out.println(JSONUtil.toJsonStr(applyInfos));
    }

    @Test
    public void saveDutyMan() {
        String dutyProject = "2c90e944668a66c001668aff5de00000,2c90e944668a66c001668b710d850002";
        String userId = UUID.randomUUID().toString().replace("-", "");

        String mobileWork = "18888888888";

        applyInfoService.saveDutyMan(dutyProject, userId, mobileWork);

        List<ApplyInfo> applyInfos = applyInfoService.fzxmList(userId);

        for (ApplyInfo applyInfo : applyInfos) {
            String fzrPhone = applyInfo.getFzrPhone();
            Assert.assertEquals(fzrPhone, mobileWork);
        }
    }

//
//    @Test
//    public void findApplyList() {
//        SearchDTO searchDTO = new SearchDTO();
//        searchDTO.setSize("10");
//        searchDTO.setPage("1");
//
//        List<ApplyInfo> applyList = applyInfoService.findApplyList(searchDTO);
//        System.out.println(JSONUtil.toJsonStr(applyList));
//
//
//    }
}